%% SignalProcessingPipeline_V1
% edits made: 02/08/21
% changed z score elimination to include >-2 and >2
% removed elimination of electrodes with "abnormal HG values"


% Preamble

% This function is intended to remove the following noises from the biosignal:
% - DC offset shift
% - linse noise
% - EMG artificats
% - Nonstationarities

% Author: Amin Nourmohammadi
% Data: Jan 19, 2021

clc
clear all
close all

%% parameters *************************************************************

% Downsampling
Setting.DownSampling.perform     = 0;    % 0 or 1
Setting.DownSampling.freq        = 400;  % downsample data to 400 Hz

% DC Offset Shift
Setting.DCoffset.perform         = 1;
Setting.DCoffset.freq            = .5;   % highpass filter above this freq
Setting.DCoffset.filterorder     = 4;

% Line Noise IIR & Notch
Setting.LineNoise.perform        = 1;    % 0 or 1
Setting.LineNoise.std            = 1.5;  % channels with line noise more than 1.5 standard deviaiton from the mean are considered noisy
Setting.LineNoise.freq           = [60,120,180]; % notch the line noise and it's harmonics
Setting.Notch.filterorder        = 4;

% Common Average Refrence (CAR)
Setting.CAR.perform              = 1;

% EMG
Setting.EMG.perform              = 1;    % 0 or 1
Setting.BandPass.filterorder     = 4;
Setting.BandPass.freqrange       = [70 170];
Setting.EMG.binSize              = 2; % second

% Nonstationarity
Setting.Nonstationarity.perform  = 1;  % 0 or 1
Setting.Nonstationarity.binSize  = .5;  % second
Setting.Nonstationarity.binThereshold    = 4;  % outlier: x std away
Setting.Nonstationarity.sampleThereshold = 4;  % outlier: x std away


%% Main Body *************************************************************

% definging in/out directories & load data

% cd('the directory that contains the main function')
addpath(genpath(pwd));

data_dir = '../Data/';
result_dir = '../Results/';

if ~exist(result_dir,'dir'); mkdir(result_dir); end

% find all of the data files to analyze
all_files_dir = dir_recursive(data_dir,'*.mat');

% we have 3 list of channels. the original channel remain the same
% throughout the processing. The other two list however, get updated after
% each analysis.
Channels.list_of_the_original_channels = [];
channels2exclude = [];
channels2analyze = [];


% loop through all the data files that are to be analyzed
for file_idx = 1:size(all_files_dir,1)
    
    file_dir = all_files_dir(file_idx).folder;
    file_name = all_files_dir(file_idx).name;
    file_dir_name = fullfile(file_dir,file_name);
    
    raw_data_struc_dir = fullfile(all_files_dir(file_idx).folder,all_files_dir(file_idx).name);
    load(raw_data_struc_dir);
    
    % since the structures within the mat files are different, for now I
    % just customized it to "PabstBaseline_mat".
    % Later we need to make the configuration of the sturcutures uniform to
    % facilitate the loading process.
    
    % SAA edit
    % For Pabst
%     SR = D.Wave.fs; % sampling rate
%     data = D.Wave.data;
    % For Killian/Rogue/Schlitz Array 1
%     SR = D.Wav1.fs;
%     data = D.Wav1.data;
    % For Killian/Rogue/Schlitz Array 2
    SR = D.Wav2.fs;
    data = D.Wav2.data;
    
    %SR = int64(SR);
    data = double(data);
    % convert data matrix to [sample X channel]
    [s1 , s2] = size(data);
    if s1<s2
        data = transpose(data);
    end
    
    Channels.list_of_the_original_channels = 1:size(data,2);
    
    channels2analyze = 1:size(data,2);
    
    % Downsampling ========================================================
    if Setting.DownSampling.perform
        
        sfrq = SR;
        dsfrq = Setting.DownSampling.freq;
        n = round(sfrq/dsfrq);
        
        fprintf('\n');
        fprintf(sprintf('>> Downsampling data from %d to %d Hz ... \n',sfrq,dsfrq));
        
        data_ds = [];
        for ch_idx = 1:size(data,2)
            %data_ds(:,ch_idx) = resample(data(:,ch_idx),dsfrq,sfrq);
            %data_ds(:,ch_idx) = downsample(data(:,ch_idx),);
        end
        
        data = data_ds;
        SR = Setting.DownSampling.freq;
    end
    
    
    % DC offset correction ================================================
    if Setting.DCoffset.perform
        
        fprintf('\n');
        fprintf(sprintf('>> Correcting DC offset shift by Highpass filtering above %d Hz ...\n',Setting.DCoffset.freq));
        
        order = Setting.DCoffset.filterorder;
        Wn    = Setting.DCoffset.freq/(SR/2);
        [b,a] = butter(order, Wn, 'high');
        flag = isstable(b,a); assert(flag == 1,'Highpass is unstable')
        data = filtfilt(b,a,data);
        
    end
    
    
    % Line noise ==========================================================
    if Setting.LineNoise.perform
        
        fprintf('\n');
        fprintf('>> Identifying channels with significant line noise ... \n')
        
        % IIR-peak filter
        peak.fcenter    = 60;
        peak.bw         = 0.001;
        peak.wo         = peak.fcenter/(SR/2);
        [peak.b,peak.a] = iirpeak(peak.wo,peak.bw);
        
        for ch_idx = 1:size(data,2)
            data_CAR   = data(:,ch_idx) - mean(data,2);
            data_LineNoise_RMS(ch_idx) = sqrt(mean(filtfilt(peak.b,peak.a,data_CAR).^2));
        end
        
        channels_LineNoise  = find(data_LineNoise_RMS > (mean(data_LineNoise_RMS) + ...
            (Setting.LineNoise.std) * std(data_LineNoise_RMS)));
        
        Channels.channels_with_signnificant_LineNoise = channels_LineNoise;
        
        channels2exclude = sort(union(channels2exclude , channels_LineNoise)) ;
        
        channels2analyze = sort(setdiff(channels2analyze , channels2exclude));
        
        data = data(: , channels2analyze);
        
        
        % Notch filter
        
        fprintf('\n');
        fprintf('>> Removing line noise : Notch filtering ... \n');
        
        fcenters = Setting.LineNoise.freq;
        
        order = Setting.Notch.filterorder;
        
        for ii = 1:length(fcenters)
            
            Wn = [fcenters(ii)-2 fcenters(ii)+2]/(SR/2);
            [b,a] = butter(order, Wn, 'stop');
            flag = isstable(b,a); assert(flag == 1,'Notch Filter is unstable')
            data = filtfilt(b,a,data);
            
        end
        
    end
    
    
    % CAR =================================================================
    if Setting.CAR.perform
        
        fprintf('\n');
        fprintf('>> Common Average Refrencing ... \n');
        
        data(:,:) = data(:,:) - repmat(mean(data,2),1,size(data,2));
        
    end
    
    
    % EMG =================================================================
    if Setting.EMG.perform
        
        fprintf('\n');
        fprintf('>> Identifying time intervals that the animal moved ... \n');
        
        % extracting high-gamma (70-170 Hz) envelop to identify
        order = Setting.BandPass.filterorder;
        Wn    = Setting.BandPass.freqrange /(SR/2);
        [b,a] = butter(order, Wn, 'bandpass');
        
        BandPassed_data = filtfilt(b,a,data);
        HighGammaEnvelop = abs(hilbert(BandPassed_data));
        
        
        bin_length = Setting.EMG.binSize * SR;
        
        
        % at this step we bin the data into the pre-determined length (eg 5 second)
        % identify the interval where at least one of the channels
        % demonstrate significant deviation from the mean of all the
        % channels over all the bins
        
        NumOfBins = floor(size(data,1)/bin_length);
        NumOfBins = int64(NumOfBins);
        
        bin_mean = [];
        
        for ii = 1:NumOfBins
            bin_mean(ii,:) = mean(HighGammaEnvelop(bin_length*(ii-1)+1 : bin_length*ii , :));
        end
        
        
        z_score_bin_mean = zscore(bin_mean, 0, 'all');
        
        % sanity check for individual channels
        % in the analysis I noticed sometime one channel shows a suspicious
        % amount of high frequency activities beyond what is expected based
        % on observing other channels. Thus such channel should be excluded
        % from the analysis.
%         channels_with_suspecisou_HG_activities_idx = find(isoutlier(...
%             sum(z_score_bin_mean>3),'percentiles',[2 98]));
%         
%         
%         if ~isempty(channels_with_suspecisou_HG_activities_idx)
%             
%             channels_with_suspecisou_HG_activities = channels2analyze(...
%                 channels_with_suspecisou_HG_activities_idx);
%             
%             Channels.channels_with_suspecisou_HG_activities = ...
%                 channels_with_suspecisou_HG_activities;
%             
%             channels2exclude = sort(union(channels2exclude,...
%                 channels_with_suspecisou_HG_activities));
%             
%             channels2analyze = setdiff(channels2analyze,...
%                 channels2exclude);
%             
%             data(:,channels_with_suspecisou_HG_activities_idx) = [];
%             
%             z_score_bin_mean(:,channels_with_suspecisou_HG_activities_idx) = [];
%             
%         end
%          bin2exclude = [];
%         for ii = 1:NumOfBins
%             if any(z_score_bin_mean(ii,:)>1) 
%                 bin2exclude = [bin2exclude ii];
%             end
%         end  
         bin2exclude = [];
        for ii = 1:NumOfBins
            if any(z_score_bin_mean(ii,:)>1.5) 
                bin2exclude = [bin2exclude ii];
            end
        end  
%         bin2exclude = [];
%         for ii = 1:NumOfBins
%             if any(z_score_bin_mean(ii,:)>1.5) || any(z_score_bin_mean(ii,:)<-1.5)
%                 bin2exclude = [bin2exclude ii];
%             end
%         end
     
        
        % now that the intervals with movements are identified we can
        % remove them from all of the channels
        for ii = bin2exclude
            initial_idx = ((ii-1)*bin_length)+1;
            last_idx = ii*bin_length;
            
            if last_idx>size(data,1)
                last_idx = size(data,1);
            end
            
            data(initial_idx:last_idx, :) = [];
        end
        
    end
    
    
    % Nonstationarity =====================================================
    if Setting.Nonstationarity.perform
        
        
        TH1 = Setting.Nonstationarity.sampleThereshold;
        if any(sum(abs(zscore(data))>TH1,2))
            noisy_sample_points = find(sum(abs(zscore(data))>TH1,2));
            data(noisy_sample_points , :) = [];
        end
            
        
        % we measure the temporal variation in data to identify channels &
        % time intervals with significant nonstationarity.
        
        bin_length = Setting.Nonstationarity.binSize * SR;
        
        NumOfBins = floor(size(data,1)/bin_length);
        NumOfBins = int64(NumOfBins);
        
        bin_mean = [];
        bin_std = [];
        
        for ii = 1:NumOfBins
            bin_mean(ii,:) = mean(data(bin_length*(ii-1)+1 : bin_length*ii , :));
            bin_std(ii,:)  = std(data(bin_length*(ii-1)+1 : bin_length*ii , :));
        end
        
        z_score_bin_mean = zscore(bin_mean);
        z_score_bin_std = zscore(bin_std);
        
        TH2 = Setting.Nonstationarity.binThereshold;
        boolean1 = abs(z_score_bin_mean)>TH2;
        boolean2 = abs(z_score_bin_std)>TH2;
        
        boolean_mix = or(boolean1,boolean2);
        
        bin2exclude = [];
        for ii = 1:NumOfBins
            if any(boolean_mix(ii,:))
                bin2exclude = [bin2exclude ii];
            end
        end
        
        % now that the intervals with movements are identified we can
        % remove them from all of the channels
        for ii = bin2exclude
            initial_idx = ((ii-1)*bin_length)+1;
            last_idx = ii*bin_length;
            
            if last_idx>size(data,1)
                last_idx = size(data,1);
            end
            
            data(initial_idx:last_idx, :) = [];
        end
        
        
    end
    
    
    
    % saving results for the individual data file ========================
    
    Channels.channels2exclude = channels2exclude;
    Channels.channels2analyze = channels2analyze;
    
    % Pabst
%     if contains(file_dir_name,'P')
%         output_file_dir = fullfile(result_dir,'PabstBaseline_v2');
%         if ~exist(output_file_dir,'dir'); mkdir(output_file_dir); end
%         output_file_name = fullfile(output_file_dir , file_name);
%         save(output_file_name,'Setting','Channels','data')
%     end
    % Killian
%      if contains(file_dir_name,'K')
%         output_file_dir = fullfile(result_dir,'KillianBaseline2_v2');
%         if ~exist(output_file_dir,'dir'); mkdir(output_file_dir); end
%         output_file_name = fullfile(output_file_dir , file_name);
%         save(output_file_name,'Setting','Channels','data')
%     end   
    % Rogue
%       if contains(file_dir_name,'R')
%         output_file_dir = fullfile(result_dir,'RogueBaseline1_v2');
%         if ~exist(output_file_dir,'dir'); mkdir(output_file_dir); end
%         output_file_name = fullfile(output_file_dir , file_name);
%         save(output_file_name,'Setting','Channels','data')
%       end     
    % Schlitz
      if contains(file_dir_name,'S')
        output_file_dir = fullfile(result_dir,'SchlitzBaseline_a2_v2');
        if ~exist(output_file_dir,'dir'); mkdir(output_file_dir); end
        output_file_name = fullfile(output_file_dir , file_name);
        save(output_file_name,'Setting','Channels','data')
      end      
end

% EOF



%% supplementary

% plotting all time-traces
% figure;
% for ii=1:size(data,2)   
%     dd=data(:,ii)/max(data(:,ii));
%     plot(dd+(ii*2)); hold on
% end
% axis tight


% plotting the spectrogram
% X = data(:,20);
% Fs = SR;
% figure;
% subplot 311
% plot(X);   axis tight
% xlabel('Sample');
% [pxx,f]=pwelch(X,[],[],[],Fs);
% subplot 312
% plot(f,(pxx)); xlim([0,200])
% xlabel('Frequency (Hz)');
% ylabel('Power');
% subplot 313
% plot(f,pow2db(pxx)); xlim([0,200])
% xlabel('Frequency (Hz)');
% ylabel('Power (dB)');

