%% SignalProcessingPipeline_V2

% Preamble

% This is the SignalProcessingPipeline version 2. The modification that has
% been made in version 2 are the following:
% 1- creating a for loop to include all the four animals
% 2- change the "data directory" to read the data files from the source
%   folders within the "Old_NHP_ECoG" folder in the Box, rather than
%   the "Data" folder inside the "Analysis_Package"
% 3- Unifying the "channels2exlude" list by concatenating all the data
%    files for each animal and analyze them together at once.

% Note:
% 1- It seems all the data files have the same sampling rate (2.0345e+03)
% 2- It seems all the data files have the same original dimension which is
%    channel X sample point. Thus, we transpose them to facillitate the
%    analysis.

% Author: Amin Nourmohammadi
% Data: March 3, 2021

clc
clear
close all

%% parameters *************************************************************


% Note:
% all data files have the same sampling rate (SR): (2.0345e+03)
SR = 2035;


Setting.Unifying_channels2exlude.perform = 1;

% Downsampling
Setting.DownSampling.perform     = 1;    % 0 or 1
Setting.DownSampling.freq        = 400;  % downsample data to 400 Hz

% DC Offset Shift
Setting.DCoffset.perform         = 1;
Setting.DCoffset.freq            = 1;   % highpass filter above this freq
Setting.DCoffset.filterorder     = 4;

% Line Noise IIR & Notch
Setting.LineNoise.perform        = 1;    % 0 or 1
Setting.LineNoise.std            = 1.5;  % channels with line noise more than 1.5 standard deviaiton from the mean are considered noisy
Setting.LineNoise.freq           = [60,120,180]; % notch the line noise and it's harmonics
Setting.Notch.filterorder        = 4;

% Common Average Refrence (CAR)
Setting.CAR.perform              = 1;

% EMG
Setting.EMG.perform              = 1;    % 0 or 1
Setting.BandPass.filterorder     = 4;
Setting.BandPass.freqrange       = [70 170];
Setting.EMG.binSize              = 2; % second
Setting.EMG.zscoreThreshold      = 1.5; % if the mean of z-scored HG env is larger than this then consider that bin (time interval) as moving

% Nonstationarity
Setting.Nonstationarity.perform  = 1;  % 0 or 1
Setting.Nonstationarity.binSize  = 1;  % second
Setting.Nonstationarity.binThereshold    = 1.5;  % outlier: x std away
Setting.Nonstationarity.sampleThereshold = 2;  % outlier: x std away
Setting.Nonstationarity.excludingChannelsThereshold = 2;


%% definging in/out directories

% change Matlab current directory to the directory that contains this main
% function. Note that all the directories will be relative to the "Script"
% direcotry.

% cd('')

addpath(genpath(fullfile('..','.')));

% general directories for data and results
data_dir = fullfile('..','Data');
result_dir = fullfile('..','Results');

Setting.data_dir = data_dir;
Setting.result_dir = result_dir;

% verify the data directory exist
if ~exist(data_dir,'dir')
    error('Data directory was not found!')
end

% creating the result directory
if ~exist(result_dir,'dir')
    mkdir(result_dir);
end


%% *********************************************************************
% specifying which animal and which data file to analyze
% the code will analyze any animal that is mentione in this list:

%Setting.subject_list = {'Killian' , 'Pabst' , 'Rogue' , 'Schlitz'};
Setting.subject_list = {'Rogue'};

% specifiying which data files to be analyzed.
%If no file is specified here then the code goes through all of the data files
%
% Example:
% for instance if we only want to analyze the two data files for "Killina" then we specify their name like this:
% Setting.subject.files2analyze.Killian = {'K20120307_before','K20120730-baseline-1'};

Setting.subject.files2analyze.Killian = {};
Setting.subject.files2analyze.Pabst = {};
Setting.subject.files2analyze.Rogue = {};
Setting.subject.files2analyze.Schlitz = {};
% *********************************************************************


for sub_idx = 1:length(Setting.subject_list)
    
    sub_name = Setting.subject_list{sub_idx};
    
    fprintf('\n')
    fprintf(sprintf('Preprocessing %s:',sub_name))
    
    % subject-specific directories
    % data dir
    sub_data_dir = dir(fullfile(data_dir,sprintf('*%s*',sub_name)));
    sub_data_dir = fullfile(data_dir,sub_data_dir.name);
    % result dir
    sub_result_dir = fullfile(result_dir,sprintf('%s',sub_name));
    if ~exist(sub_result_dir,'dir'); mkdir(sub_result_dir); end    
    
    
    dummy_var = fullfile(sprintf('%s',sub_result_dir),'Channels.mat');
    
    if Setting.Unifying_channels2exlude.perform
        
        fprintf('\n')
        fprintf('Unifying_channels2exlude:')
        [Channels] = Unifying_Channels2Exclude(Setting,sub_name);
        
    else
        % load the result of Unifying_Channels2Exclude from previous runs
        load(dummy_var)
        
    end
    
    
    
    % specifying the targe files to be analyzed for the for loop over data files
    dummy_var = eval(sprintf('Setting.subject.files2analyze.%s',sub_name));
    if ~isempty(dummy_var)
        
        sub_data_files = dummy_var;
        
    else
        
        % find all the data files within the directory
        clear sub_data_files
        sub_data_files_dummy = dir(fullfile(sub_data_dir,'*.mat'));        
        for ii = 1:length(sub_data_files_dummy)
            sub_data_files{ii} = sub_data_files_dummy(ii).name;
        end
        
        
    end
    
    
    % for loop over specified data files
    for file_idx = 1:length(sub_data_files)
        
        file_name = sub_data_files{file_idx};
        file_dir = fullfile(sub_data_dir , file_name);
        
        % loading data files
        if strcmp(sub_name,'Pabst')            
            
            load(file_dir)
            % only keep the good channels (i.e. channels2analyze)
            DATA{1} = double(transpose(D.Wave.data(Channels{1}.channels2analyze , :)));
            
        else           
            % all other animals have the same data structure (wav1 & wav2)  
            load(file_dir)
            % only keep the good channels (i.e. channels2analyze)
            DATA{1} = double(transpose(D.Wav1.data(Channels{1}.channels2analyze , :)));
            DATA{2} = double(transpose(D.Wav2.data(Channels{2}.channels2analyze , :)));
            
        end
        
        
        % process the data corresponding to each existing array seperately: 
        
        for array_idx = 1:length(DATA)
            
            data = DATA{array_idx};
            
            % Downsampling ========================================================
            if Setting.DownSampling.perform                
               
                n = round(SR/Setting.DownSampling.freq);               
                data = downsample(data,n);
                SR = Setting.DownSampling.freq;
                
            end
            
            
            % CAR =========================================================
            if Setting.CAR.perform
                
                fprintf('\n');
                fprintf('>> Common Average Refrencing ... \n');
                
                data(:,:) = data(:,:) - repmat(mean(data,2),1,size(data,2));
                
            end
            
            
            
            % EMG Artifact ================================================
            
            if Setting.EMG.perform
                
                fprintf('\n');
                fprintf('>> Identifying time intervals that animal moved ... \n');
                
                % extracting high-gamma (70-170 Hz) envelop to identify
                order = Setting.BandPass.filterorder;
                Wn    = Setting.BandPass.freqrange /(SR/2);
                [b,a] = butter(order, Wn, 'bandpass');eegplot(temp_data', 'srate', 1200)
                
                BandPassed_data = filtfilt(b,a,data);
                HighGammaEnvelop = abs(hilbert(BandPassed_data));
                
                
                bin_length = Setting.EMG.binSize * SR;
                
                
                % at this step we bin the data into the pre-determined length (eg 5 second)
                % identify the interval where at least one of the channels
                % demonstrate significant deviation from the mean of all the
                % channels over all the bins
                
                NumOfBins = floor(size(data,1)/bin_length);
                NumOfBins = int64(NumOfBins);
                
                bin_mean = [];
                
                for ii = 1:NumOfBins
                    bin_mean(ii,:) = mean(HighGammaEnvelop(bin_length*(ii-1)+1 : bin_length*ii , :));
                end
                
                z_score_bin_mean = zscore(bin_mean, 0, 'all');
                
                bin2exclude = [];
                for ii = 1:NumOfBins
                    if any(abs(z_score_bin_mean(ii,:))>Setting.EMG.zscoreThreshold)
                        bin2exclude = [bin2exclude ii];
                    end
                end
                
                
                % now that the intervals with movements are identified we can
                % remove them from all of the channels
                for ii = bin2exclude
                    initial_idx = ((ii-1)*bin_length)+1;
                    last_idx = ii*bin_length;
                    
                    if last_idx>size(data,1)
                        last_idx = size(data,1);
                    end
                    
                    data(initial_idx:last_idx, :) = [];
                end
                
            end
            
            
            
            % Nonstationarity =================================================
            if Setting.Nonstationarity.perform
                
                
                %             TH1 = Setting.Nonstationarity.sampleThereshold;
                %             if any(sum(abs(zscore(data))>TH1,2))
                %                 noisy_sample_points = find(sum(abs(zscore(data))>TH1,2));
                %                 data(noisy_sample_points , :) = [];
                %             end
                
                
                % we measure the temporal variation in data to identify channels &
                % time intervals with significant nonstationarity.
                
                bin_length = Setting.Nonstationarity.binSize * SR;
                
                NumOfBins = floor(size(data,1)/bin_length);
                NumOfBins = int64(NumOfBins);
                
                % zscoring each channel seperately to make the signal in
                % each channel zero-meaned
                data_zscored = zscore(data);
                
                bin_mean = [];
                bin_std = [];
                
                for ii = 1:NumOfBins
                    bin_mean(ii,:) = mean(data_zscored(bin_length*(ii-1)+1 : bin_length*ii , :));
                    bin_std(ii,:)  = std(data_zscored(bin_length*(ii-1)+1 : bin_length*ii , :));
                end
                
                z_score_bin_mean = zscore(bin_mean);
                z_score_bin_std = zscore(bin_std);
                
                TH2 = Setting.Nonstationarity.binThereshold;
                boolean1 = abs(z_score_bin_mean)>TH2;
                boolean2 = abs(z_score_bin_std)>TH2;
                
                boolean_mix = or(boolean1,boolean2);
                
                bin2exclude = [];
                for ii = 1:NumOfBins
                    if any(boolean_mix(ii,:))
                        bin2exclude = [bin2exclude ii];
                    end
                end
                
                % now that the intervals with nonstationarity are identified
                % we can remove them from all of the channels
                for ii = bin2exclude
                    initial_idx = ((ii-1)*bin_length)+1;
                    last_idx = ii*bin_length;
                    
                    if last_idx>size(data,1)
                        last_idx = size(data,1);
                    end
                    
                    data(initial_idx:last_idx, :) = [];
                    
                end
                
                
            end
            
            preprocessed_data{array_idx} = data;
            
        end
        
        % saving results for the individual data file =======================        
        
        output_file_dir = fullfile(sub_result_dir,file_name);
        save(output_file_dir,'Setting','preprocessed_data')
        
        
    end
close all; clear all;    
end


% EOF


