%% Unifying_Channels2Exclude

function [Channels] = Unifying_Channels2Exclude(Setting,sub_name)

% Objective:
% 1- Unifying the "channels2exlude" list by concatenating all the data
%    files for each animal and analyze them together.

% Note:
% 1- It seems all the data files have the same sampling rate (2.0345e+03)
% 2- It seems all the data files have the same original dimension which is
%    channel X sample point. Thus, we transpose them to facillitate the
%    analysis.

% all data files have the same sampling rate: (2.0345e+03)
SR = 2035;

subject_list = Setting.subject_list;

data_dir = Setting.data_dir;
result_dir = Setting.result_dir;

% identifying the number of existing data files for the subject
sub_data_dir = dir(fullfile(data_dir,sprintf('*%s*',sub_name)));
sub_data_dir = fullfile(data_dir,sub_data_dir.name);
sub_data_files = dir(fullfile(sub_data_dir,'*.mat'));


% concatenating all the data files
clear data
concatenated_data = [];
concatenated_data_array1 = [];
concatenated_data_array2 = [];

fprintf('\n')
fprintf('Concatenating data files [')

if strcmp(sub_name,'Pabst')
    
    for file_idx = 1:length(sub_data_files)
        
        file_dir = fullfile(sub_data_files(file_idx).folder , sub_data_files(file_idx).name);
        load(file_dir)
        data = double(transpose(D.Wave.data));
        data = downsample(data,round(SR/Setting.DownSampling.freq));
        concatenated_data = cat(1,concatenated_data,data);
        fprintf('.')
        
    end
    
    DATA{1} = concatenated_data;
    
else % all other animals have the same data structure (wav1 & wav2)
    
    for file_idx = 1:length(sub_data_files)
        
        file_dir = fullfile(sub_data_files(file_idx).folder , sub_data_files(file_idx).name);
        load(file_dir)
        % array 1
        data1 = double(transpose(D.Wav1.data));
        data1 = downsample(data1,round(SR/Setting.DownSampling.freq));
        concatenated_data_array1 = cat(1,concatenated_data_array1,data1);
        % array 2
        data2 = double(transpose(D.Wav2.data));
        data2 = downsample(data2,round(SR/Setting.DownSampling.freq));
        concatenated_data_array2 = cat(1,concatenated_data_array2,data2);
        
        fprintf('.')
        
    end
    
    DATA{1} = concatenated_data_array1;
    DATA{2} = concatenated_data_array2;
    
end

fprintf(']')

clear Channels

for array_idx = 1:length(DATA)
    
    data = DATA{array_idx};
    
    
    % DC offset correction ============================================
    
    fprintf('\n');
    fprintf(sprintf('>> Correcting DC offset shift by Highpass filtering above %d Hz ...\n',Setting.DCoffset.freq));
    
    order = Setting.DCoffset.filterorder;
    Wn    = Setting.DCoffset.freq/(SR/2);
    [b,a] = butter(order, Wn, 'high');
    flag = isstable(b,a); assert(flag == 1,'Highpass is unstable')
    data = filtfilt(b,a,data);
    
    
    channels2exclude = [];
    channels2analyze = 1:size(data,2);
    
    % Line noise ======================================================
    
    fprintf('\n');
    fprintf('>> Identifying channels with significant line noise ... \n')
    
    % IIR-peak filter
    peak.fcenter    = 60;
    peak.bw         = 0.001;
    peak.wo         = peak.fcenter/(SR/2);
    [peak.b,peak.a] = iirpeak(peak.wo,peak.bw);
    
    for ch_idx = 1:size(data,2)
        data_CAR   = data(:,ch_idx) - mean(data,2);
        data_LineNoise_RMS(ch_idx) = sqrt(mean(filtfilt(peak.b,peak.a,data_CAR).^2));
    end
    
    channelsWithSignificantLineNoise  = find(data_LineNoise_RMS > (mean(data_LineNoise_RMS) + ...
        (Setting.LineNoise.std) * std(data_LineNoise_RMS)));
    
    channels2exclude = sort(union(channels2exclude , ...
        channelsWithSignificantLineNoise)) ;
    
    
    % Notch filter
    fcenters = Setting.LineNoise.freq;
    order = Setting.Notch.filterorder;
    
    for ii = 1:length(fcenters)
        
        Wn = [fcenters(ii)-2 fcenters(ii)+2]/(SR/2);
        [b,a] = butter(order, Wn, 'stop');
        flag = isstable(b,a); assert(flag == 1,'Notch Filter is unstable')
        data = filtfilt(b,a,data);
        
    end
    
    
    % Nonstationarity =================================================
    
    fprintf('\n');
    fprintf('>> Identifying Nonstationary channels ... \n')
    
    % measuring the tempral variation in the signals and identify
    % channels with significant nonstationarity. we bin the signal from
    % each channel and measure the variation of mean and std across
    % bins for each channel
    
    bin_length = Setting.Nonstationarity.binSize * SR;
    
    NumOfBins = floor(size(data,1)/bin_length);
    NumOfBins = int64(NumOfBins);
    
    data = zscore(data);
    
    bin_mean = [];
    bin_std = [];
    
    for ii = 1:NumOfBins
        bin_mean(ii,:) = mean(data(bin_length*(ii-1)+1 : bin_length*ii , :));
        bin_std(ii,:)  = std(data(bin_length*(ii-1)+1 : bin_length*ii , :));
    end
    
    
    % nonstationarity (NS)
    mean_based_NS_idx = var(bin_mean);
    std_based_NS_idx = var(bin_std);
    
    % at this point we should decide which channels should be excluded
    % we can z-score the two NS_idx above and exclude any channel with
    % abs(z-score) > 3 for example.
    mean_based_NS_idx_zscore = zscore(mean_based_NS_idx);
    std_based_NS_idx_zscore = zscore(std_based_NS_idx);
    
    channelsWithSignificantNonstationarity1 = find(abs(mean_based_NS_idx_zscore)>Setting.Nonstationarity.excludingChannelsThereshold);
    channelsWithSignificantNonstationarity2 = find(abs(std_based_NS_idx_zscore)>Setting.Nonstationarity.excludingChannelsThereshold);
    
    channels2exclude = sort(union(channels2exclude , ...
        channelsWithSignificantNonstationarity1));
    
    channels2exclude = sort(union(channels2exclude , ...
        channelsWithSignificantNonstationarity2));
    
    channels2analyze = sort(setdiff(channels2analyze , channels2exclude));
    
    % merging variable
    Channels{array_idx}.channels2exclude = channels2exclude;
    Channels{array_idx}.channels2analyze = channels2analyze;
    
    
end

% saving the output as "Channels.mat"
output_dir = fullfile(result_dir,sprintf('%s',sub_name), 'Channels.mat');
save(output_dir,'Channels')





%% Additional steps to identify more bad channels
% % sanity check for individual channels
% % in the analysis I noticed sometime one channel shows a suspicious
% % amount of high frequency activities beyond what is expected based
% % on observing other channels. Thus such channel should be excluded
% % from the analysis.
% channels_with_suspecisou_HG_activities_idx = find(isoutlier(...
%     sum(z_score_bin_mean>3),'percentiles',[2 98]));
%
%
% if ~isempty(channels_with_suspecisou_HG_activities_idx)
%
%     channels_with_suspecisou_HG_activities = channels2analyze(...
%         channels_with_suspecisou_HG_activities_idx);
%
%     Channels.channels_with_suspecisou_HG_activities = ...
%         channels_with_suspecisou_HG_activities;
%
%     channels2exclude = sort(union(channels2exclude,...
%         channels_with_suspecisou_HG_activities));
%
%     channels2analyze = setdiff(channels2analyze,...
%         channels2exclude);
%
%     data(:,channels_with_suspecisou_HG_activities_idx) = [];
%
%     z_score_bin_mean(:,channels_with_suspecisou_HG_activities_idx) = [];
%
% end

%[EOF]


